@@ -1,8 +1,8 @@
-from ivoryos.server import run
+from ivoryos.server import run, global_config
 from ivoryos.optimizer.registry import OPTIMIZER_REGISTRY
 from ivoryos.version import __version__ as ivoryos_version
 from ivoryos.utils.decorators import block, BUILDING_BLOCKS
-from ivoryos.app import app
+from ivoryos.app import app, create_app, socketio, db
 
 __all__ = [
     "block",
@@ -11,4 +11,8 @@ __all__ = [
     "run",
     "app",
     "ivoryos_version",
+    "create_app",
+    "socketio",
+    "global_config",
+    "db"
 ]
