@@ -23,7 +23,7 @@ control.register_blueprint(control_temp)
 @control.route("/", strict_slashes=False, methods=["GET", "POST"])
 @control.route("/<string:instrument>", strict_slashes=False, methods=["GET", "POST"])
 @login_required
-def deck_controllers(instrument: str = None):
+async def deck_controllers(instrument: str = None):
     """
     .. :quickref: Direct Control; device (instruments) and methods
 
@@ -52,7 +52,10 @@ def deck_controllers(instrument: str = None):
         inst_object = find_instrument_by_name(instrument)
         if instrument.startswith("blocks"):
             forms = create_form_from_pseudo(pseudo=inst_object, autofill=False, design=False)
+        elif instrument.startswith("deck"):
+            forms = create_form_from_pseudo(pseudo=global_config.deck_snapshot[instrument], autofill=False, design=False)
         else:
+            #TODO
             forms = create_form_from_module(sdl_module=inst_object, autofill=False, design=False)
         order = get_session_by_instrument('card_order', instrument)
         hidden_functions = get_session_by_instrument('hidden_functions', instrument)
@@ -78,11 +81,23 @@ def deck_controllers(instrument: str = None):
         if request.is_json:
             kwargs = {k: v for k, v in payload.items() if k not in ["csrf_token", "hidden_wait"]}
         else:
-            kwargs = {field.name: field.data for field in form if field.name not in ["csrf_token", "hidden_name"]}
+            if not form.validate_on_submit():
+                flash(f"Run Error! {form.errors}", "error")
+                return render_template(
+                    "controllers.html",
+                    defined_variables=global_config.deck_snapshot.keys(),
+                    block_variables=global_config.building_blocks.keys(),
+                    temp_variables=global_config.defined_variables.keys(),
+                    instrument=instrument,
+                    forms=forms,
+                    session=session
+                )
+            else:
+                kwargs = {field.name: field.data for field in form if field.name not in ["csrf_token", "hidden_name"]}
 
         wait = str(payload.get("hidden_wait", "true")).lower() == "true"
 
-        output = runner.run_single_step(
+        output = await runner.run_single_step(
             component=instrument, method=method_name, kwargs=kwargs, wait=wait,
             current_app=current_app._get_current_object()
         )
