@@ -87,8 +87,11 @@ def experiment_builder():
 
     # edit_action_info = session.get("edit_action")
 
-
-    exec_string = script.python_script if script.python_script else script.compile(current_app.config['SCRIPT_FOLDER'])
+    try:
+        exec_string = script.python_script if script.python_script else script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        flash(f"Error in Python script: {e}")
     session['python_code'] = exec_string
 
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
@@ -97,6 +100,27 @@ def experiment_builder():
                            script=script, defined_variables=deck_variables, buttons_dict=design_buttons,
                            local_variables=global_config.defined_variables, block_variables=global_config.building_blocks)
 
+@design.route("/draft/code_preview", methods=["GET"])
+@login_required
+def compile_preview():
+    # Get mode and batch from query parameters
+    script = utils.get_script_file()
+    mode = request.args.get("mode", "single")   # default to "single"
+    batch = request.args.get("batch", "sample") # default to "sample"
+
+    try:
+        # Example: decide which code to return based on mode/batch
+        if mode == "single":
+            code = script.compile(current_app.config['SCRIPT_FOLDER'])
+        elif mode == "batch":
+            code = script.compile(current_app.config['SCRIPT_FOLDER'], batch=True, mode=batch)
+        else:
+            code = "Invalid mode. Please select 'single' or 'batch'."
+    except Exception as e:
+        code = f"Error compiling: {e}"
+    # print(code)
+    return jsonify(code=code)
+
 
 @design.route("/draft/meta", methods=["PATCH"])
 @login_required
@@ -316,25 +340,43 @@ def methods_handler(instrument: str = ''):
     msg = ""
     request.form
     if "hidden_name" in request.form:
+        deck_snapshot = global_config.deck_snapshot
+        block_snapshot = global_config.building_blocks
         method_name = request.form.get("hidden_name", None)
         form = forms.get(method_name) if forms else None
         insert_position = request.form.get("drop_target_id", None)
+
         if form:
             kwargs = {field.name: field.data for field in form if field.name != 'csrf_token'}
+            # print(kwargs)
             if form.validate_on_submit():
                 function_name = kwargs.pop("hidden_name")
+                batch_action = kwargs.pop("batch_action", False)
                 save_data = kwargs.pop('return', '')
+
+                # validate return variable name
+                save_data = script.validate_function_name(save_data)
+
                 primitive_arg_types = utils.get_arg_type(kwargs, functions[function_name])
 
                 # todo
                 # print(primitive_arg_types)
 
                 script.eval_list(kwargs, primitive_arg_types)
-                kwargs = script.validate_variables(kwargs)
+                kwargs = script.validate_variables(kwargs, primitive_arg_types)
+                coroutine = False
+                if instrument.startswith("deck") and deck_snapshot:
+                    coroutine = deck_snapshot[instrument][function_name].get("coroutine", False)
+                elif instrument.startswith("blocks") and block_snapshot:
+                    coroutine = block_snapshot[instrument][function_name].get("coroutine", False)
+                # print(kwargs)
                 action = {"instrument": instrument, "action": function_name,
                           "args": kwargs,
                           "return": save_data,
-                          'arg_types': primitive_arg_types}
+                          'arg_types': primitive_arg_types,
+                          "coroutine": coroutine,
+                          "batch_action": batch_action,
+                          }
                 script.add_action(action=action, insert_position=insert_position)
             else:
                 msg = [f"{field}: {', '.join(messages)}" for field, messages in form.errors.items()]
@@ -350,9 +392,15 @@ def methods_handler(instrument: str = ''):
                 if 'variable' in kwargs:
                     try:
                         script.add_variable(insert_position=insert_position, **kwargs)
-                    except ValueError:
+                    except ValueError as e:
                         success = False
-                        msg = [f"{field}: {', '.join(messages)}" for field, messages in form.errors.items()]
+                        msg = e.__str__()
+                elif 'math_variable' in kwargs:
+                    try:
+                        script.add_math_variable(insert_position=insert_position, **kwargs)
+                    except ValueError as e:
+                        success = False
+                        msg = e.__str__()
                 else:
                     script.add_logic_action(logic_type=logic_type, insert_position=insert_position, **kwargs)
             else:
@@ -366,9 +414,13 @@ def methods_handler(instrument: str = ''):
             kwargs = {field.name: field.data for field in form if field.name != 'csrf_token'}
             if form.validate_on_submit():
                 save_data = kwargs.pop('return', '')
+
+                # validate return variable name
+                save_data = script.validate_function_name(save_data)
+
                 primitive_arg_types = utils.get_arg_type(kwargs, functions[workflow_name])
                 script.eval_list(kwargs, primitive_arg_types)
-                kwargs = script.validate_variables(kwargs)
+                kwargs = script.validate_variables(kwargs, primitive_arg_types)
                 action = {"instrument": instrument, "action": workflow_name,
                           "args": kwargs,
                           "return": save_data,
@@ -379,7 +431,13 @@ def methods_handler(instrument: str = ''):
                 success = False
                 msg = [f"{field}: {', '.join(messages)}" for field, messages in form.errors.items()]
     utils.post_script_file(script)
-    exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    #TODO
+    try:
+        exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        msg = f"Compilation failed: {str(e)}"
+    # exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
     session['python_code'] = exec_string
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
     html = render_template("components/canvas_main.html", script=script, buttons_dict=design_buttons)
