@@ -23,12 +23,16 @@ def get_step(uuid: int):
     """
     script = utils.get_script_file()
     action = script.find_by_uuid(uuid)
-    if request.method == 'GET':
-        # forms = create_form_from_action(action, script=script)
+    if action is None:
+        return jsonify({"warning": "Step not found, please refresh the page."}), 404
+
+    elif request.method == 'GET':
+        forms = create_form_from_action(action, script=script)
         # session['edit_action'] = action
         return render_template("components/edit_action_form.html",
                                action=action,
-                               forms=create_form_from_action(action, script=script))
+                               forms=forms)
+
 
 
 @steps.post("/draft/steps/<int:uuid>")
@@ -47,22 +51,36 @@ def save_step(uuid: int):
     """
     script = utils.get_script_file()
     action = script.find_by_uuid(uuid)
+    warning = None
     if action is not None:
         forms = create_form_from_action(action, script=script)
         kwargs = {field.name: field.data for field in forms if field.name != 'csrf_token'}
         if forms and forms.validate_on_submit():
-            save_as = kwargs.pop('return', '')
-            kwargs = script.validate_variables(kwargs)
-            script.update_by_uuid(uuid=uuid, args=kwargs, output=save_as)
+            save_data = kwargs.pop('return', '')
+            # validate return variable name
+            save_data = script.validate_function_name(save_data)
+
+            batch_action = kwargs.pop('batch_action', False)
+
+            # literal for args with no typehint
+            arg_types = action.get('arg_types', {})
+            kwargs = script.validate_variables(kwargs, arg_types)
+
+            script.update_by_uuid(uuid=uuid, args=kwargs, output=save_data, batch_action=batch_action)
         else:
-            flash(forms.errors)
+            warning = f"Compilation failed: {str(forms.errors)}"
     utils.post_script_file(script)
-    exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    try:
+        exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        warning = f"Compilation failed: {str(e)}"
     session['python_code'] = exec_string
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
     return render_template("components/canvas_main.html",
-                               script=script,
-                               buttons_dict=design_buttons)
+                           script=script,
+                           buttons_dict=design_buttons,
+                           warning=warning)
 
 @steps.delete("/draft/steps/<int:uuid>")
 def delete_step(uuid: int):
@@ -82,12 +100,17 @@ def delete_step(uuid: int):
     if request.method == 'DELETE':
         script.delete_action(uuid)
     utils.post_script_file(script)
-    exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    warning = None
+    try:
+        exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        warning = f"Compilation failed: {str(e)}"
     session['python_code'] = exec_string
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
     return render_template("components/canvas_main.html",
                                script=script,
-                               buttons_dict=design_buttons)
+                               buttons_dict=design_buttons, warning=warning)
 
 
 @steps.route("/draft/steps/<int:uuid>/duplicate", methods=["POST"], strict_slashes=False,)
@@ -107,13 +130,18 @@ def duplicate_action(uuid: int):
     script = utils.get_script_file()
     script.duplicate_action(uuid)
     utils.post_script_file(script)
-    exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    warning = None
+    try:
+        exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        warning = f"Compilation failed: {str(e)}"
     session['python_code'] = exec_string
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
 
     return render_template("components/canvas_main.html",
                          script=script,
-                         buttons_dict=design_buttons)
+                         buttons_dict=design_buttons, warning=warning)
 
 
 @steps.route("/draft/steps/order", methods=['POST'])
@@ -133,13 +161,18 @@ def update_list():
     script = utils.get_script_file()
     script.currently_editing_order = order.split(",", len(script.currently_editing_script))
     script.sort_actions()
+    warning = None
 
     utils.post_script_file(script)
-    exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    try:
+        exec_string = script.compile(current_app.config['SCRIPT_FOLDER'])
+    except Exception as e:
+        exec_string = {}
+        warning = f"Compilation failed: {str(e)}"
     session['python_code'] = exec_string
 
     # Return the updated canvas HTML instead of JSON
     design_buttons = {stype: create_action_button(script, stype) for stype in script.stypes}
     return render_template("components/canvas_main.html",
                            script=script,
-                           buttons_dict=design_buttons)
\ No newline at end of file
+                           buttons_dict=design_buttons, warning=warning)
\ No newline at end of file
