@@ -3,7 +3,7 @@ import os
 import time
 
 from flask import Blueprint, redirect, url_for, flash, jsonify, request, render_template, session, \
-    current_app, g
+    current_app, g, send_file
 from flask_login import login_required
 
 from ivoryos.routes.execute.execute_file import files
@@ -76,14 +76,17 @@ def experiment_run():
         if isinstance(exec_string, dict):
             for key, func_str in exec_string.items():
                 exec(func_str)
-            line_collection = script.convert_to_lines(exec_string)
+
         else:
             # Handle string case - you might need to adjust this based on your needs
-            line_collection = []
+            line_collection = {}
     except Exception:
         flash(f"Please check syntax!!")
         return redirect(url_for("design.experiment_builder"))
 
+
+    line_collection = script.render_script_lines(script.script_dict)
+
     run_name = script.name if script.name else "untitled"
 
     dismiss = session.get("dismiss", None)
@@ -92,8 +95,13 @@ def experiment_run():
 
     _, return_list = script.config_return()
     config_list, config_type_list = script.config("script")
-    data_list = os.listdir(current_app.config['DATA_FOLDER'])
-    data_list.remove(".gitkeep") if ".gitkeep" in data_list else data_list
+    data_list = [f for f in os.listdir(current_app.config['DATA_FOLDER']) if f.endswith('.csv')]
+    # Remove .gitkeep if present
+    if ".gitkeep" in data_list:
+        data_list.remove(".gitkeep")
+
+    # Sort by creation time, newest first
+    data_list.sort(key=lambda f: os.path.getctime(os.path.join(current_app.config['DATA_FOLDER'], f)), reverse=True)
 
     if deck is None:
         no_deck_warning = True
@@ -105,31 +113,36 @@ def experiment_run():
             flash(f"This script is not compatible with current deck, import {script.deck}")
 
     if request.method == "POST":
-        bo_args = None
+        # bo_args = None
         compiled = False
         if request.accept_mimetypes.best_match(['application/json', 'text/html']) == 'application/json':
             payload_json = request.get_json()
             compiled = True
             if "kwargs" in payload_json:
                 config = payload_json["kwargs"]
-            elif "parameters" in payload_json:
-                bo_args = payload_json
+            # elif "parameters" in payload_json:
+            #     bo_args = payload_json
             repeat = payload_json.pop("repeat", None)
+            batch_size = payload_json.pop('batch_size', 1)
         else:
             if "bo" in request.form:
                 bo_args = request.form.to_dict()
                 existing_data = bo_args.pop("existing_data")
             if "online-config" in request.form:
-                config = utils.web_config_entry_wrapper(request.form.to_dict(), config_list)
+                config_args = request.form.to_dict()
+                config_args.pop("batch_size", None)
+                config = utils.web_config_entry_wrapper(config_args, config_list)
+            batch_size = int(request.form.get('batch_size', 1))
             repeat = request.form.get('repeat', None)
 
         try:
+        # if True:
             datapath = current_app.config["DATA_FOLDER"]
             run_name = script.validate_function_name(run_name)
-            runner.run_script(script=script, run_name=run_name, config=config, bo_args=bo_args,
+            runner.run_script(script=script, run_name=run_name, config=config,
                               logger=g.logger, socketio=g.socketio, repeat_count=repeat,
                               output_path=datapath, compiled=compiled, history=existing_data,
-                              current_app=current_app._get_current_object()
+                              current_app=current_app._get_current_object(), batch_size=batch_size
                               )
             if utils.check_config_duplicate(config):
                 flash(f"WARNING: Duplicate in config entries.")
@@ -152,6 +165,22 @@ def experiment_run():
                                no_deck_warning=no_deck_warning, dismiss=dismiss, design_buttons=design_buttons,
                                history=deck_list, pause_status=runner.pause_status(), optimizer_schema=optimizers_schema)
 
+@execute.route("/executions/optimizer_schema", methods=["POST"])
+def optimizer_schema():
+    if request.accept_mimetypes.best_match(['application/json', 'text/html']) == 'application/json':
+        payload_json = request.get_json()
+        optimizer_type = payload_json.pop("optimizer_type", None)
+        if optimizer_type:
+            _schema = global_config.optimizers.get(optimizer_type, None)
+            if _schema is None:
+                return jsonify({"error": f"Optimizer {optimizer_type} is not supported or not found."})
+            return jsonify(_schema.get_schema())
+        else:
+            optimizers_schema = {k: v.get_schema() for k, v in global_config.optimizers.items()}
+            return jsonify(optimizers_schema)
+    return None
+
+
 @execute.route("/executions/campaign", methods=["POST"])
 @login_required
 def run_bo():
@@ -172,23 +201,61 @@ def run_bo():
     """
     script = utils.get_script_file()
     run_name = script.name if script.name else "untitled"
-    payload = request.form.to_dict()
-    repeat = payload.pop("repeat", None)
-    optimizer_type = payload.pop("optimizer_type", None)
-    existing_data = payload.pop("existing_data", None)
-    parameters, objectives, steps = parse_optimization_form(payload)
+
+    if request.accept_mimetypes.best_match(['application/json', 'text/html']) == 'application/json':
+        payload_json = request.get_json()
+        objectives = payload_json.pop("objectives", None)
+        parameters = payload_json.pop("parameters", None)
+        steps = payload_json.pop("steps", None)
+        constraints = payload_json.pop("parameter_constraints", None)
+        repeat = payload_json.pop("repeat", None)
+        batch_size = payload_json.pop("batch_size", None)
+        optimizer_type = payload_json.pop("optimizer_type", None)
+        existing_data = payload_json.pop("existing_data", None)
+        additional_params = payload_json.pop("additional_params", None)
+    else:
+        payload = request.form.to_dict()
+        repeat = payload.pop("repeat", None)
+        optimizer_type = payload.pop("optimizer_type", None)
+        existing_data = payload.pop("existing_data", None)
+
+        # Handle file upload if present
+        if 'uploaded_data' in request.files:
+            file = request.files['uploaded_data']
+            if file and file.filename and file.filename.endswith('.csv'):
+                filename = secure_filename(file.filename)
+                filepath = os.path.join(current_app.config['DATA_FOLDER'], filename)
+                file.save(filepath)
+                existing_data = filename
+                
+        batch_mode = payload.pop("batch_mode", None)
+        batch_size = payload.pop("batch_size", 1)
+
+        # Get constraint expressions (new single-line input)
+        constraint_exprs = request.form.getlist("constraint_expr")
+        constraints = [expr.strip() for expr in constraint_exprs if expr.strip()]
+
+        # Remove constraint_expr entries from payload before parsing parameters
+        for key in list(payload.keys()):
+            if key.startswith("constraint_expr"):
+                payload.pop(key, None)
+
+        parameters, objectives, steps, additional_params = parse_optimization_form(payload)
+        # print(additional_params)
+    # if True:
     try:
         datapath = current_app.config["DATA_FOLDER"]
         run_name = script.validate_function_name(run_name)
         Optimizer = global_config.optimizers.get(optimizer_type, None)
         if not Optimizer:
             raise ValueError(f"Optimizer {optimizer_type} is not supported or not found.")
-        optimizer = Optimizer(experiment_name=run_name, parameter_space=parameters, objective_config=objectives,
-                              optimizer_config=steps)
-        runner.run_script(script=script, run_name=run_name, optimizer=optimizer,
+
+        runner.run_script(script=script, run_name=run_name, optimizer=None,
                           logger=g.logger, socketio=g.socketio, repeat_count=repeat,
                           output_path=datapath, compiled=False, history=existing_data,
-                          current_app=current_app._get_current_object()
+                          current_app=current_app._get_current_object(), batch_size=int(batch_size),
+                          objectives=objectives, parameters=parameters, constraints=constraints, steps=steps,
+                          optimizer_cls=Optimizer, additional_params=additional_params
                           )
 
     except Exception as e:
@@ -198,6 +265,21 @@ def run_bo():
             flash(e.__str__())
     return redirect(url_for("execute.experiment_run"))
 
+@execute.route("/executions/latest_plot")
+@login_required
+def get_optimizer_plot():
+
+    optimizer = current_app.config.get("LAST_OPTIMIZER")
+    if optimizer is not None:
+        # the placeholder is for showing different plots
+        latest_file = optimizer.get_plots('placeholder')
+        # print(latest_file)
+        if files:
+            return send_file(latest_file, mimetype="image/png")
+    # print("No plots found")
+    return jsonify({"error": "No plots found"}), 404
+
+
 
 
 @execute.route("/executions/status", methods=["GET"])
