@@ -27,6 +27,9 @@ class User(db.Model, UserMixin):
     # email = db.Column(db.String)
     hashPassword = db.Column(db.String(255))
 
+    # New columns for logo customization
+    settings = db.Column(JSONType, nullable=True)
+
     # password = db.Column()
     def __init__(self, username, password):
         # self.id = id
@@ -50,11 +53,13 @@ class Script(db.Model):
     id_order = db.Column(JSONType, nullable=True)
     editing_type = db.Column(db.String(50), nullable=True)
     author = db.Column(db.String(50), nullable=False)
+    description = db.Column(db.String(255), nullable=True)
     # registered = db.Column(db.Boolean, nullable=True, default=False)
 
     def __init__(self, name=None, deck=None, status=None, script_dict: dict = None, id_order: dict = None,
                  time_created=None, last_modified=None, editing_type=None, author: str = None,
                  # registered:bool=False,
+                 description: str = None,
                  python_script: str = None
                  ):
         if script_dict is None:
@@ -73,7 +78,8 @@ class Script(db.Model):
             last_modified = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
         if editing_type is None:
             editing_type = "script"
-
+        if description is None:
+            description = ""
         self.name = name
         self.deck = deck
         self.status = status
@@ -84,6 +90,7 @@ class Script(db.Model):
         self.editing_type = editing_type
         self.author = author
         self.python_script = python_script
+        self.description = description
         # self.r = registered
 
     def as_dict(self):
@@ -122,7 +129,7 @@ class Script(db.Model):
                 pass
         raise TypeError(f"Input type error: cannot convert '{args}' to {arg_type}.")
 
-    def update_by_uuid(self, uuid, args, output):
+    def update_by_uuid(self, uuid, args, output, batch_action=False):
         action = self.find_by_uuid(uuid)
         if not action:
             return
@@ -134,6 +141,7 @@ class Script(db.Model):
             pass
         action['args'] = args
         action['return'] = output
+        action['batch_action'] = batch_action
 
     @staticmethod
     def eval_list(args, arg_types):
@@ -195,11 +203,11 @@ class Script(db.Model):
                         # print(i+1)
                         action['id'] = i + 1
                         break
-            self.id_order[script_type].sort()
+            self.id_order[script_type].sort(key=int)
             if not int(self.id_order[script_type][-1]) == len(self.script_dict[script_type]):
                 new_order = list(range(1, len(self.script_dict[script_type]) + 1))
                 self.id_order[script_type] = [str(i) for i in new_order]
-            self.script_dict[script_type].sort(key=lambda x: x['id'])
+            self.script_dict[script_type].sort(key=lambda x: int(x['id']))
 
     def sort_actions(self, script_type=None):
         if script_type:
@@ -217,15 +225,33 @@ class Script(db.Model):
         self._insert_action(insert_position, current_len)
         self.update_time_stamp()
 
-    def add_variable(self, statement, variable, type, insert_position=None):
+    def add_variable(self, statement, variable, variable_type, insert_position=None):
         variable = self.validate_function_name(variable)
-        convert_type = getattr(builtins, type)
-        statement = convert_type(statement)
+        convert_type = getattr(builtins, variable_type)
+        if isinstance(statement, str) and statement.startswith("#"):
+            pass
+        # boolean values
+        elif variable_type == "bool":
+            statement = True if statement.lower() in ["true", "y", "t", "yes"] else False
+        else:
+            statement = convert_type(statement)
         current_len = len(self.currently_editing_script)
         uid = uuid.uuid4().fields[-1]
         action = {"id": current_len + 1, "instrument": 'variable', "action": variable,
                         "args": {"statement": 'None' if statement == '' else statement}, "return": '', "uuid": uid,
-                        "arg_types": {"statement": type}}
+                        "arg_types": {"statement": variable_type}}
+        self.currently_editing_script.append(action)
+        self._insert_action(insert_position, current_len)
+        self.update_time_stamp()
+
+    def add_math_variable(self, statement, math_variable, insert_position=None):
+        math_variable = self.validate_function_name(math_variable)
+
+        current_len = len(self.currently_editing_script)
+        uid = uuid.uuid4().fields[-1]
+        action = {"id": current_len + 1, "instrument": 'math_variable', "action": math_variable,
+                        "args": {"statement": 'None' if statement == '' else statement}, "return": '', "uuid": uid,
+                        "arg_types": {"statement": 'float'}}
         self.currently_editing_script.append(action)
         self._insert_action(insert_position, current_len)
         self.update_time_stamp()
@@ -242,8 +268,10 @@ class Script(db.Model):
     def get_added_variables(self):
         added_variables: Dict[str, str] = {action["action"]: action["arg_types"]["statement"] for action in
                                            self.currently_editing_script if action["instrument"] == "variable"}
-
-        return added_variables
+        added_math_variables: Dict[str, str] = {action["action"]: action["arg_types"]["statement"] for action in
+                                                self.currently_editing_script if action["instrument"] == "math_variable"}
+        all_added_variables = {**added_variables, **added_math_variables}
+        return all_added_variables
 
     def get_output_variables(self):
         output_variables: Dict[str, str] = {action["return"]: "function_output" for action in
@@ -258,7 +286,7 @@ class Script(db.Model):
 
         return output_variables
 
-    def validate_variables(self, kwargs):
+    def validate_variables(self, kwargs, arg_types: dict = None):
         """
         Validates the kwargs passed to the Script
         """
@@ -268,17 +296,19 @@ class Script(db.Model):
             if isinstance(value, str):
                 if value in output_variables:
                     var_type = output_variables[value]
-                    kwargs[key] = {value: var_type}
+                    kwargs[key] = f"#{value}"
                 elif value.startswith("#"):
                     kwargs[key] = f"#{self.validate_function_name(value[1:])}"
                 else:
                     # attempt to convert to numerical or bool value for args with no type hint
-                    try:
-                        converted = ast.literal_eval(value)
-                        if isinstance(converted, (int, float, bool)):
-                            kwargs[key] = converted
-                    except (ValueError, SyntaxError):
-                        pass
+                    type_hint = arg_types.get(key, "") if arg_types else ""
+                    if not type_hint:
+                        try:
+                            converted = ast.literal_eval(value)
+                            if isinstance(converted, (int, float, bool)):
+                                kwargs[key] = converted
+                        except (ValueError, SyntaxError):
+                            pass
         return kwargs
 
     def add_logic_action(self, logic_type: str, statement, insert_position=None):
@@ -370,28 +400,41 @@ class Script(db.Model):
         :return: list of variable that require input
         """
         configure = []
+        variables = self.get_variables()
+        # print(variables)
         config_type_dict = {}
         for action in self.script_dict[stype]:
             args = action['args']
             if args is not None:
                 if type(args) is not dict:
-                    if type(args) is str and args.startswith("#") and not args[1:] in configure:
-                        configure.append(args[1:])
-                        config_type_dict[args[1:]] = action['arg_types']
+                    if type(args) is str and args.startswith("#"):
+                        key = args[1:]
+                        if key not in (*variables, *configure):
+                            configure.append(key)
+                            config_type_dict[key] = action['arg_types']
 
                 else:
-                    for arg in args:
-                        if type(args[arg]) is str \
-                                and args[arg].startswith("#") \
-                                and not args[arg][1:] in configure:
-                            configure.append(args[arg][1:])
-                            if arg in action['arg_types']:
-                                if action['arg_types'][arg] == '':
-                                    config_type_dict[args[arg][1:]] = "any"
-                                else:
-                                    config_type_dict[args[arg][1:]] = action['arg_types'][arg]
-                            else:
-                                config_type_dict[args[arg][1:]] = "any"
+                    if action['instrument'] == "math_variable":
+                        # assume any kind of math variable will be evaluated to float even if it might be an int
+                        pattern = r"#([A-Za-z_][A-Za-z0-9_]*)"
+                        vars_found = re.findall(pattern, args['statement'])
+                        for key in vars_found:
+                            if key not in (*variables, *configure):
+                                configure.append(key)
+                                config_type_dict[key] = action['arg_types']['statement']
+                    else:
+                        for arg in args:
+                            if type(args[arg]) is str and args[arg].startswith("#"):
+                                key = args[arg][1:]
+                                if key not in (*variables, *configure):
+                                    configure.append(key)
+                                    if arg in action['arg_types']:
+                                        if action['arg_types'][arg] == '':
+                                            config_type_dict[key] = "any"
+                                        else:
+                                            config_type_dict[key] = action['arg_types'][arg]
+                                    else:
+                                        config_type_dict[key] = "any"
         # todo
         return configure, config_type_dict
 
@@ -402,7 +445,7 @@ class Script(db.Model):
         """
 
         return_list = set([action['return'] for action in self.script_dict['script'] if not action['return'] == ''])
-        output_str = "return {"
+        output_str = "{"
         for i in return_list:
             output_str += "'" + i + "':" + i + ","
         output_str += "}"
@@ -434,17 +477,115 @@ class Script(db.Model):
         :return: A dict containing script types as keys and lists of function body lines as values.
         """
         line_collection = {}
+
         for stype, func_str in exec_str_collection.items():
             if func_str:
                 module = ast.parse(func_str)
-                func_def = next(node for node in module.body if isinstance(node, ast.FunctionDef))
 
-                # Extract function body as source lines
-                line_collection[stype] = [ast_unparse(node) for node in func_def.body if not isinstance(node, ast.Return)]
-                # print(line_collection[stype])
+                # Find the first function (regular or async)
+                func_def = next(
+                    node for node in module.body
+                    if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef))
+                )
+
+                # Extract function body as source lines, skipping 'return' nodes
+                line_collection[stype] = [
+                    ast_unparse(node) for node in func_def.body if not isinstance(node, ast.Return)
+                ]
         return line_collection
 
-    def compile(self, script_path=None):
+    def render_script_lines(self, script_dict):
+        """
+        Convert the script_dict structure into a dict of displayable Python-like lines,
+        keeping ID consistency for highlighting.
+        """
+
+        def render_args(args):
+            if not args:
+                return ""
+            return ", ".join(f"{k}={v}" for k, v in args.items())
+
+        def parse_block(block):
+            lines = []
+            indent = 0
+            stack = []
+
+            for action in block:
+                act = action["action"]
+                _id = action["id"]
+                instrument = action["instrument"]
+
+                # Handle control structures
+                if act == "if":
+                    stmt = action["args"].get("statement", "")
+                    lines.append("    " * indent + f"if {stmt}:")
+                    indent += 1
+                    stack.append("if")
+
+                elif act == "else":
+                    indent -= 1
+                    lines.append("    " * indent + f"else:")
+                    indent += 1
+
+                elif act in ("endif", "endwhile"):
+                    if stack:
+                        stack.pop()
+                    indent = max(indent - 1, 0)
+                    lines.append("    " * indent + f"# {act}")
+
+                elif act == "while":
+                    stmt = action["args"].get("statement", "")
+                    lines.append("    " * indent + f"while {stmt}:")
+                    indent += 1
+                    stack.append("while")
+
+                elif act == "wait":
+                    stmt = action["args"].get("statement", "")
+                    if isinstance(stmt, str) and stmt.startswith("#"):
+                        stmt = stmt[1:]
+                    lines.append("    " * indent + f"time.sleep({stmt})")
+
+                elif instrument == "variable":
+                    stmt = action["args"].get("statement", "")
+                    if isinstance(stmt, str) and stmt.startswith("#"):
+                        stmt = stmt[1:]
+                    lines.append("    " * indent + f"{act} = {stmt}")
+
+                elif instrument == "math_variable":
+                    stmt = action["args"].get("statement", "")
+                    lines.append("    " * indent + f"{act} = {stmt}")
+
+
+                else:
+                    # Regular function call
+                    instr = action["instrument"]
+                    args = render_args(action.get("args", {}))
+                    ret = action.get("return")
+                    line = "    " * indent
+                    if ret:
+                        line += f"{ret} = {instr}.{act}({args})"
+                    else:
+                        line += f"{instr}.{act}({args})"
+                    lines.append(line)
+
+            # Ensure empty control blocks get "pass"
+            final_lines = []
+            for i, line in enumerate(lines):
+                final_lines.append(line)
+                # if line.strip().startswith("else") and (
+                #         i == len(lines) - 1 or lines[i + 1].startswith("#") or "endif" in lines[i + 1]
+                # ):
+                    # final_lines.append("    " * (indent) + "pass")
+
+            return final_lines
+
+        return {
+            "prep": parse_block(script_dict.get("prep", [])),
+            "script": parse_block(script_dict.get("script", [])),
+            "cleanup": parse_block(script_dict.get("cleanup", []))
+        }
+
+    def compile(self, script_path=None, batch=False, mode="sample"):
         """
         Compile the current script to a Python file.
         :return: String to write to a Python file.
@@ -459,7 +600,8 @@ class Script(db.Model):
 
         for i in self.stypes:
             if self.script_dict[i]:
-                func_str = self._generate_function_header(run_name, i) + self._generate_function_body(i)
+                is_async = any(a.get("coroutine", False) for a in self.script_dict[i])
+                func_str = self._generate_function_header(run_name, i, is_async, batch) + self._generate_function_body(i, batch, mode)
                 exec_str_collection[i] = func_str
         if script_path:
             self._write_to_file(script_path, run_name, exec_str_collection)
@@ -477,50 +619,70 @@ class Script(db.Model):
             name += '_'
         return name
 
-    def _generate_function_header(self, run_name, stype):
+    def _generate_function_header(self, run_name, stype, is_async, batch=False):
         """
         Generate the function header.
         """
         configure, config_type = self.config(stype)
-
         configure = [param + f":{param_type}" if not param_type == "any" else param for param, param_type in
                      config_type.items()]
 
         script_type = f"_{stype}" if stype != "script" else ""
-        function_header = f"def {run_name}{script_type}("
+        async_str = "async " if is_async else ""
+        function_header = f"{async_str}def {run_name}{script_type}("
 
         if stype == "script":
-            function_header += ", ".join(configure)
-
+            if batch:
+                function_header += "param_list" if configure else "n: int"
+            else:
+                function_header += ", ".join(configure)
         function_header += "):"
-        # function_header += self.indent(1) + f"global {run_name}_{stype}"
+
+        if stype == "script" and batch:
+            function_header += self.indent(1) + f'"""Batch mode is experimental and may have bugs."""'
         return function_header
 
-    def _generate_function_body(self, stype):
+    def _generate_function_body(self, stype, batch=False, mode="sample"):
         """
         Generate the function body for each type in stypes.
         """
         body = ''
         indent_unit = 1
+        if batch and stype == "script":
+            return_str, return_list = self.config_return()
+            configure, config_type = self.config(stype)
+            if not configure:
+                body += self.indent(indent_unit) + "param_list = [{} for _ in range(n)]"
+            for index, action in enumerate(self.script_dict[stype]):
+                text, indent_unit = self._process_action(indent_unit, action, index, stype, batch)
+                body += text
+            if return_list:
+                # body += self.indent(indent_unit) + f"result_list.append({return_str})"
+                body += self.indent(indent_unit) + "return param_list"
 
-        for index, action in enumerate(self.script_dict[stype]):
-            text, indent_unit = self._process_action(indent_unit, action, index, stype)
-            body += text
-        return_str, return_list = self.config_return()
-        if return_list and stype == "script":
-            body += self.indent(indent_unit) + return_str
+        else:
+            for index, action in enumerate(self.script_dict[stype]):
+                text, indent_unit = self._process_action(indent_unit, action, index, stype)
+                body += text
+            return_str, return_list = self.config_return()
+            if return_list and stype == "script":
+                body += self.indent(indent_unit) + f"return {return_str}"
         return body
 
-    def _process_action(self, indent_unit, action, index, stype):
+    def _process_action(self, indent_unit, action, index, stype, batch=False, mode="sample"):
         """
         Process each action within the script dictionary.
         """
+        configure, config_type = self.config(stype)
+
         instrument = action['instrument']
         statement = action['args'].get('statement')
         args = self._process_args(action['args'])
 
         save_data = action['return']
         action_name = action['action']
+        batch_action = action.get("batch_action", False)
+
         next_action = self._get_next_action(stype, index)
         # print(args)
         if instrument == 'if':
@@ -528,19 +690,46 @@ class Script(db.Model):
         elif instrument == 'while':
             return self._process_while(indent_unit, action_name, statement, next_action)
         elif instrument == 'variable':
-            return self.indent(indent_unit) + f"{action_name} = {statement}", indent_unit
+            if batch:
+                if isinstance(statement, str) and statement.startswith("#"):
+                    return '', indent_unit
+                return self.indent(indent_unit) + "for param in param_list:" + self.indent(
+                    indent_unit + 1) + f"param['{action_name}'] = {statement}", indent_unit
+            else:
+                if isinstance(statement, str) and statement.startswith("#"):
+                    statement = statement[1:]
+                return self.indent(indent_unit) + f"{action_name} = {statement}", indent_unit
         elif instrument == 'wait':
+            if isinstance(statement, str) and statement.startswith("#"):
+                statement = statement[1:]
+            if batch:
+                return f"{self.indent(indent_unit)}for param in param_list:" + f"{self.indent(indent_unit+1)}time.sleep({statement})", indent_unit
             return f"{self.indent(indent_unit)}time.sleep({statement})", indent_unit
         elif instrument == 'repeat':
             return self._process_repeat(indent_unit, action_name, statement, next_action)
         elif instrument == 'pause':
             self.needs_call_human = True
-            return f"{self.indent(indent_unit)}pause('{statement}')", indent_unit
-        #todo
+            if isinstance(statement, str) and statement.startswith("#"):
+                statement = f"str({statement[1:]})"
+            else:
+                # cases like "text with a backslash \"
+                statement = statement.encode('unicode_escape').decode()
+            if batch:
+                return f"{self.indent(indent_unit)}for param in param_list:" + f"{self.indent(indent_unit+1)}pause('''{statement}''')", indent_unit
+            return f"{self.indent(indent_unit)}pause('''{statement}''')", indent_unit
+        elif instrument == "math_variable":
+            math_expression = self._process_math(statement)
+            if batch:
+                return f"{self.indent(indent_unit)}for param in param_list:" + f"{self.indent(indent_unit + 1)}param['{action_name}'] = {math_expression}", indent_unit
+            else:
+                return f"{self.indent(indent_unit)}{action_name} = {math_expression}", indent_unit
+        # todo
         # elif instrument == 'registered_workflows':
         #     return inspect.getsource(my_function)
         else:
-            return self._process_instrument_action(indent_unit, instrument, action_name, args, save_data)
+            is_async = action.get("coroutine", False)
+            dynamic_arg = len(self.get_variables()) > 0
+            return self._process_instrument_action(indent_unit, instrument, action_name, args, save_data, is_async, dynamic_arg, batch, batch_action)
 
     def _process_args(self, args):
         """
@@ -556,6 +745,8 @@ class Script(db.Model):
         """
         exec_string = ""
         if action == 'if':
+            if isinstance(args, str) and args.startswith("#"):
+                args = args[1:]
             exec_string += self.indent(indent_unit) + f"if {args}:"
             indent_unit += 1
             if next_action and next_action['instrument'] == 'if' and next_action['action'] == 'else':
@@ -578,6 +769,8 @@ class Script(db.Model):
         """
         exec_string = ""
         if action == 'while':
+            if isinstance(args, str) and args.startswith("#"):
+                args = args[1:]
             exec_string += self.indent(indent_unit) + f"while {args}:"
             indent_unit += 1
             if next_action and next_action['instrument'] == 'while':
@@ -591,6 +784,8 @@ class Script(db.Model):
         Process 'while' and 'endwhile' actions.
         """
         exec_string = ""
+        if isinstance(args, str) and args.startswith("#"):
+            args = args[1:]
         if action == 'repeat':
             exec_string += self.indent(indent_unit) + f"for _ in range({args}):"
             indent_unit += 1
@@ -600,10 +795,21 @@ class Script(db.Model):
             indent_unit -= 1
         return exec_string, indent_unit
 
-    def _process_instrument_action(self, indent_unit, instrument, action, args, save_data):
+    def _process_math(self, expr: str) -> str:
+        """
+        Return the math expression as a string but remove '#' prefix from variable names
+        """
+        # remove leading "#" for variable tokens
+        cleaned = re.sub(r"#([A-Za-z_]\w*)", r"\1", expr)
+        return cleaned
+
+    def _process_instrument_action(self, indent_unit, instrument, action, args, save_data, is_async=False, dynamic_arg=False,
+                                   batch=False, batch_action=False):
         """
         Process actions related to instruments.
         """
+        async_str = "await " if is_async else ""
+
         function_call = f"{instrument}.{action}"
         if instrument.startswith("blocks"):
             self.blocks_included = True
@@ -611,16 +817,30 @@ class Script(db.Model):
 
         if isinstance(args, dict) and args != {}:
             args_str = self._process_dict_args(args)
-            single_line = f"{function_call}(**{args_str})"
+            single_line = f"{async_str}{function_call}(**{args_str})"
         elif isinstance(args, str):
             single_line = f"{function_call} = {args}"
         else:
-            single_line = f"{function_call}()"
+            single_line = f"{async_str}{function_call}()"
 
-        if save_data:
-            save_data += " = "
 
-        return self.indent(indent_unit) + save_data + single_line, indent_unit
+        save_data_str = save_data + " = " if save_data else ''
+
+        if batch and not batch_action:
+            arg_list = [args[arg][1:] for arg in args if isinstance(args[arg], str) and args[arg].startswith("#")]
+            param_str = [f"param['{arg_list}']" for arg_list in arg_list if arg_list]
+            args_str = self.indent(indent_unit + 1) +  ", ".join(arg_list) + " = " + ", ".join(param_str) if arg_list else ""
+            if dynamic_arg:
+                for_string = self.indent(indent_unit) + "for param in param_list:" + args_str
+            else:
+                for_string = self.indent(indent_unit) + "for i in range(n):"
+            output_code = for_string + self.indent(indent_unit + 1) + save_data_str + single_line
+            if save_data:
+                output_code = output_code + self.indent(indent_unit + 1) + f"param['{save_data}'] = {save_data}"
+        else:
+            output_code = self.indent(indent_unit) + save_data_str + single_line
+
+        return output_code, indent_unit
 
     def _process_dict_args(self, args):
         """
@@ -632,12 +852,21 @@ class Script(db.Model):
                 args_str = args_str.replace(f"'#{args[arg][1:]}'", args[arg][1:])
             elif isinstance(args[arg], dict):
                 # print(args[arg])
-                variables = self.get_variables()
+                if not args[arg]:
+                    continue
+                # Extract the variable name (first key in the dict)
                 value = next(iter(args[arg]))
-                if value not in variables:
-                    raise ValueError(f"Variable ({value}) is not defined.")
-                args_str = args_str.replace(f"{args[arg]}", next(iter(args[arg])))
-            # elif self._is_variable(arg):
+                var_type = args[arg].get(value)
+
+                # Only process if it's a function_output variable reference
+                if var_type == "function_output":
+                    variables = self.get_variables()
+                    if value not in variables:
+                        raise ValueError(f"Variable ({value}) is not defined.")
+                    # Replace the dict string representation with just the variable name
+                    args_str = args_str.replace(f"{args[arg]}", value)
+
+        # elif self._is_variable(arg):
             #     print("is variable")
             #     args_str = args_str.replace(f"'{args[arg]}'", args[arg])
         return args_str
@@ -654,7 +883,7 @@ class Script(db.Model):
         """
         Check if the argument is of type 'variable'.
         """
-        return arg in self.script_dict and self.script_dict[arg].get("arg_types") == "variable"
+        return arg in self.script_dict and self.script_dict[arg].get("arg_types") in ("variable", 'math_variable')
 
     def _write_to_file(self, script_path, run_name, exec_string, call_human=False):
         """
