@@ -1,4 +1,4 @@
-from enum import Enum
+from enum import Enum, EnumMeta
 from typing import Union, Any
 try:
     from typing import get_origin, get_args
@@ -51,8 +51,8 @@ class VariableOrStringField(Field):
             variable, variable_type = find_variable(self.data, self.script)
             if variable:
                 return variable
-
-        return str(self.data) if self.data is not None else ""
+        # print("_value", self.data, type(self.data))
+        return str(self.data) if self.data else ''
 
 
 class VariableOrFloatField(Field):
@@ -141,6 +141,10 @@ class VariableOrIntField(Field):
                 raise ValueError(self.gettext("Variable is not supported in prep/cleanup"))
             self.data = valuelist[0]
             return
+        if valuelist[0] == "":
+            # print("empty input", valuelist)
+            self.data = None
+            return
         try:
             self.data = int(valuelist[0])
         except ValueError as exc:
@@ -148,9 +152,9 @@ class VariableOrIntField(Field):
             raise ValueError(self.gettext("Not a valid integer value.")) from exc
 
 
-class VariableOrBoolField(BooleanField):
+class VariableOrBoolField(Field):
     widget = TextInput()
-    false_values = (False, "false", "", "False", "f", "F")
+    false_values = (False, "false", "", "False", "f", "F", "n")
 
     def __init__(self, label='', validators=None, script=None, **kwargs):
         super(VariableOrBoolField, self).__init__(label, validators, **kwargs)
@@ -164,8 +168,14 @@ class VariableOrBoolField(BooleanField):
                 if not variable_type == "function_output":
                     raise ValueError("Not accepting boolean variables")
                 return variable
-
-        self.data = bool(value)
+        if isinstance(value, str) and value.startswith("#"):
+            self.data = value
+            return value
+        if value in self.false_values:
+            self.data = False
+        else:
+            self.data = True
+        return None
 
     def process_formdata(self, valuelist):
         # todo
@@ -192,21 +202,41 @@ class VariableOrBoolField(BooleanField):
 
         if self.raw_data:
             return str(self.raw_data[0])
-        return "y"
+        return str(self.data)
 
 
 class FlexibleEnumField(StringField):
     def __init__(self, label=None, validators=None, choices=None, script=None, **kwargs):
         super().__init__(label, validators, **kwargs)
         self.script = script
-        self.enum_class = choices
+        self.enum_class = self._resolve_enum(choices)
         self.choices = [e.name for e in self.enum_class]
         # self.value_list = [e.name for e in self.enum_class]
 
+    def _resolve_enum(self, annotation):
+        """Extract Enum from Enum or Optional[Enum]"""
+        # Case: direct Enum
+        if isinstance(annotation, EnumMeta):
+            return annotation
+
+        # Case: Optional / Union
+        origin = get_origin(annotation)
+        if origin is Union:
+            for arg in get_args(annotation):
+                if isinstance(arg, EnumMeta):
+                    return arg
+
+        raise TypeError(f"FlexibleEnumField expected Enum or Optional[Enum], got {annotation!r}")
 
     def process_formdata(self, valuelist):
+        # todo right now enum types will be processed to methods as a str, so the method must convert to the enum type
         if valuelist:
             key = valuelist[0]
+            # Treat empty string or "None" as null value
+            if key in ("", None, "None"):
+                self.data = None
+                return
+
             if key in self.choices:
                 # Convert the string key to Enum instance
                 self.data = self.enum_class[key].value
@@ -282,10 +312,10 @@ def create_form_for_method(method, autofill, script=None, design=True):
             "validators": [InputRequired()] if param.default is param.empty else [Optional()],
             **({"script": script} if (autofill or design) else {})
         }
-        if isinstance(param.annotation, type) and issubclass(param.annotation, Enum):
-            # enum_class = [(e.name, e.value) for e in param.annotation]
+        if _is_enum_type(param.annotation):
+            enum_class = _unwrap_enum_type(param.annotation)
             field_class = FlexibleEnumField
-            placeholder_text = f"Choose or type a value for {param.annotation.__name__} (start with # for custom)"
+            placeholder_text = f"Choose or type a value for {enum_class.__name__} (start with # for custom)"
 
             extra_kwargs = {"choices": param.annotation}
 
@@ -317,6 +347,41 @@ def create_form_for_method(method, autofill, script=None, design=True):
     return DynamicForm
 
 
+def _is_enum_type(tp):
+    # Optional[Enum] comes through as Union[Enum, NoneType]
+    origin = get_origin(tp)
+
+    # Non-Optional direct enum
+    if isinstance(tp, type) and issubclass(tp, Enum):
+        return True
+
+    # Optional/Union case → unwrap inner types
+    if origin is Union:
+        return any(
+            isinstance(arg, type) and issubclass(arg, Enum)
+            for arg in get_args(tp)
+            if arg is not type(None)
+        )
+
+    return False
+
+def _unwrap_enum_type(tp):
+    from typing import get_origin, get_args, Union
+    from enum import Enum
+
+    # Bare Enum
+    if isinstance(tp, type) and issubclass(tp, Enum):
+        return tp
+
+    # Optional/Union conversion
+    origin = get_origin(tp)
+    if origin is Union:
+        for arg in get_args(tp):
+            if arg is not type(None) and isinstance(arg, type) and issubclass(arg, Enum):
+                return arg
+
+    return None
+
 def create_add_form(attr, attr_name, autofill: bool, script=None, design: bool = True):
     """
     Create forms for each method or signature
@@ -328,11 +393,15 @@ def create_add_form(attr, attr_name, autofill: bool, script=None, design: bool =
     """
     signature = attr.get('signature', {})
     docstring = attr.get('docstring', "")
+    return_type = signature.return_annotation
     # print(signature, docstring)
     dynamic_form = create_form_for_method(signature, autofill, script, design)
     if design:
-        return_value = StringField(label='Save value as', render_kw={"placeholder": "Optional"})
-        setattr(dynamic_form, 'return', return_value)
+        if return_type is not None:
+            return_value = StringField(label='Save value as', render_kw={"placeholder": "Optional"})
+            setattr(dynamic_form, 'return', return_value)
+        batch_action = BooleanField(label='Batch Action', render_kw={"placeholder": "Optional"})
+        setattr(dynamic_form, 'batch_action', batch_action)
     hidden_method_name = HiddenField(name=f'hidden_name', description=docstring, render_kw={"value": f'{attr_name}'})
     setattr(dynamic_form, 'hidden_name', hidden_method_name)
     return dynamic_form
@@ -348,13 +417,16 @@ def create_form_from_module(sdl_module, autofill: bool = False, script=None, des
     """
     method_forms = {}
     for attr_name in dir(sdl_module):
-        method = getattr(sdl_module, attr_name)
-        if inspect.ismethod(method) and not attr_name.startswith('_'):
-            signature = inspect.signature(method)
-            docstring = inspect.getdoc(method)
-            attr = dict(signature=signature, docstring=docstring)
-            form_class = create_add_form(attr, attr_name, autofill, script, design)
-            method_forms[attr_name] = form_class()
+        try:
+            method = getattr(sdl_module, attr_name)
+            if inspect.ismethod(method) and not attr_name.startswith('_'):
+                signature = inspect.signature(method)
+                docstring = inspect.getdoc(method)
+                attr = dict(signature=signature, docstring=docstring)
+                form_class = create_add_form(attr, attr_name, autofill, script, design)
+                method_forms[attr_name] = form_class()
+        except Exception as e:
+            print(f"Error creating form for {attr_name}: {e}")
     return method_forms
 
 
@@ -391,6 +463,7 @@ def create_form_from_action(action: dict, script=None, design=True):
     arg_types = action.get("arg_types", {})
     args = action.get("args", {})
     save_as = action.get("return")
+    instrument = action.get("instrument")
 
     class DynamicForm(FlaskForm):
         pass
@@ -405,19 +478,35 @@ def create_form_from_action(action: dict, script=None, design=True):
     for name, param_type in arg_types.items():
         # formatted_param_name = format_name(name)
         value = args.get(name, "")
-        if type(value) is dict:
+        if type(value) is dict and value:
             value = next(iter(value))
+        if value in (None, "", "None"):
+            value = None
+
         field_kwargs = {
             "label": name,
-            "default": f'{value}',
-            "validators": [InputRequired()],
+            "default": value,
+            # todo get optional/required from snapshot
+            "validators": [],
+            "filters": [lambda x: x if x != '' else None],
             **({"script": script})
         }
+        if type(param_type) is list:
+            none_type = param_type[1]
+            if none_type == "NoneType":
+                param_type = param_type[0]
         param_type = param_type if type(param_type) is str else f"{param_type}"
         field_class, placeholder_text = annotation_mapping.get(
             param_type,
             (VariableOrStringField if design else StringField, f'Enter {param_type} value')
         )
+
+        if instrument == "math_variable":
+            field_class = VariableOrStringField
+            placeholder_text = "Enter math expression"
+            field_kwargs["validators"] = [InputRequired()]
+
+
         render_kwargs = {"placeholder": placeholder_text}
 
         # Create the field with additional rendering kwargs for placeholder text
@@ -425,13 +514,16 @@ def create_form_from_action(action: dict, script=None, design=True):
         setattr(DynamicForm, name, field)
 
     if design:
+        if "batch_action" in action:
+            batch_action = BooleanField(label='Batch Action', default=bool(action["batch_action"]))
+            setattr(DynamicForm, 'batch_action', batch_action)
         return_value = StringField(label='Save value as', default=f"{save_as}", render_kw={"placeholder": "Optional"})
         setattr(DynamicForm, 'return', return_value)
     return DynamicForm()
 
 def create_all_builtin_forms(script):
     all_builtin_forms = {}
-    for logic_name in ['if', 'while', 'variable', 'wait', 'repeat', 'pause']:
+    for logic_name in ['if', 'while', 'variable', 'wait', 'repeat', 'pause', 'math']:
         # signature = info.get('signature', {})
         form_class = create_builtin_form(logic_name, script)
         all_builtin_forms[logic_name] = form_class()
@@ -447,18 +539,20 @@ def create_builtin_form(logic_type, script):
     placeholder_text = {
         'wait': 'Enter second',
         'repeat': 'Enter an integer',
-        'pause': 'Human Intervention Message'
+        'pause': 'Human Intervention Message',
+        'math': 'Enter math expression, e.g. #x + 5 * (#y - 2)'
     }.get(logic_type, 'Enter statement')
     description_text = {
         'variable': 'Your variable can be numbers, boolean (True or False) or text ("text")',
+        'math': "Enter a math expression using #variables, numbers, or returned variables",
     }.get(logic_type, '')
     field_class = {
         'wait': VariableOrFloatField,
-        'repeat': VariableOrIntField
+        'repeat': VariableOrIntField,
     }.get(logic_type, VariableOrStringField)  # Default to StringField as a fallback
     field_kwargs = {
         "label": f'statement',
-        "validators": [InputRequired()] if logic_type in ['wait', "variable"] else [],
+        "validators": [InputRequired()] if logic_type in ['wait', "variable", "math"] else [],
         "description": description_text,
         "script": script
     }
@@ -466,16 +560,27 @@ def create_builtin_form(logic_type, script):
     field = field_class(**field_kwargs, render_kw=render_kwargs)
     setattr(BuiltinFunctionForm, "statement", field)
     if logic_type == 'variable':
-        variable_field = StringField(label=f'variable', validators=[InputRequired()],
+        variable_field = VariableOrStringField(label=f'variable', validators=[InputRequired()],
                                      description="Your variable name cannot include space",
-                                     render_kw=render_kwargs)
+                                     render_kw=render_kwargs, script=script)
         type_field = SelectField(
             'Select Input Type',
             choices=[('int', 'Integer'), ('float', 'Float'), ('str', 'String'), ('bool', 'Boolean')],
-            default='str'  # Optional default value
+            default='str',  # Optional default value
+            # coerce = lambda x: None if x == "None" else x
         )
         setattr(BuiltinFunctionForm, "variable", variable_field)
-        setattr(BuiltinFunctionForm, "type", type_field)
+        setattr(BuiltinFunctionForm, "variable_type", type_field)
+    elif logic_type == "math":
+        math_variable_field = VariableOrStringField(
+            label="save_to",
+            validators=[InputRequired()],
+            description="Variable name to save the result into",
+            render_kw={"placeholder": "Result variable name"},
+            script=script
+        )
+        setattr(BuiltinFunctionForm, "math_variable", math_variable_field)
+
     hidden_field = HiddenField(name=f'builtin_name', render_kw={"value": f'{logic_type}'})
     setattr(BuiltinFunctionForm, "builtin_name", hidden_field)
     return BuiltinFunctionForm
@@ -537,14 +642,16 @@ def _action_button(action: dict, variables: dict):
     """
     style = {
         "repeat": "background-color: lightsteelblue",
-        "if": "background-color: salmon",
-        "while": "background-color: salmon",
-        "pause": "background-color: goldenrod",
+        "if": "background-color: mistyrose",
+        "while": "background-color: #a8b5a2",
+        "pause": "background-color: palegoldenrod",
     }.get(action['instrument'], "")
+    if not style:
+        style = "background-color: thistle" if 'batch_action' in action and action["batch_action"] else ""
 
     if action['instrument'] in ['if', 'while', 'repeat']:
         text = f"{action['action']} {action['args'].get('statement', '')}"
-    elif action['instrument'] == 'variable':
+    elif action['instrument'] in ('variable', 'math_variable'):
         text = f"{action['action']} = {action['args'].get('statement')}"
     else:
         # regular action button
@@ -556,9 +663,13 @@ def _action_button(action: dict, variables: dict):
                 arg_list = []
                 for k, v in action['args'].items():
                     if isinstance(v, dict):
-                        value = next(iter(v))  # Extract the first key if it's a dict
-                        # show warning color for variable calling when there is no definition
-                        style = "background-color: khaki" if value not in variables.keys() else ""
+                        if not v:
+                            value = v  # Keep the original value if not a dict
+                        else:
+                            value = next(iter(v))  # Extract the first key if it's a dict
+                            # show warning color for variable calling when there is no definition
+
+                            style = "background-color: khaki" if v.get(value) == "function_output" and value not in variables.keys() else ""
                     else:
                         value = v  # Keep the original value if not a dict
                     arg_list.append(f"{k} = {value}")  # Format the key-value pair
