@@ -1,6 +1,7 @@
 import ast
 import importlib
 import inspect
+import json
 import logging
 import os
 import pickle
@@ -8,6 +9,7 @@ import socket
 import subprocess
 import sys
 from collections import Counter
+from enum import Enum
 
 import flask
 from flask import session
@@ -105,7 +107,8 @@ def _inspect_class(class_object=None, debug=False):
             try:
                 annotation = inspect.signature(method)
                 docstring = inspect.getdoc(method)
-                functions[function] = dict(signature=annotation, docstring=docstring)
+                coroutine = inspect.iscoroutinefunction(method)
+                functions[function] = dict(signature=annotation, docstring=docstring, coroutine=coroutine,)
 
             except Exception:
                 pass
@@ -114,13 +117,17 @@ def _inspect_class(class_object=None, debug=False):
 
 def _get_type_from_parameters(arg, parameters):
     """get argument types from inspection"""
+    # TODO
     arg_type = ''
-    if type(parameters) is inspect.Signature:
+    if isinstance(parameters, inspect.Signature):
         annotation = parameters.parameters[arg].annotation
-    elif type(parameters) is dict:
+    elif isinstance(parameters, dict):
         annotation = parameters[arg]
-    if annotation is not inspect._empty:
-        # print(p[arg].annotation)
+    else:
+        annotation = ''
+    if isinstance(annotation, str):
+        arg_type = annotation
+    elif annotation is not inspect._empty:
         if annotation.__module__ == 'typing':
 
             if hasattr(annotation, '__origin__'):
@@ -180,6 +187,7 @@ def convert_config_type(args, arg_types, is_class: bool = False):
     Converts an argument from str to an arg type
     """
     if args:
+        Script.eval_list(args, arg_types)
         for arg in args:
             if arg not in arg_types.keys():
                 raise ValueError("config file format not supported.")
@@ -191,7 +199,7 @@ def convert_config_type(args, arg_types, is_class: bool = False):
                 arg_type = arg_types[arg]
                 try:
                     args[arg] = ast.literal_eval(args[arg])
-                except ValueError:
+                except Exception:
                     pass
                 if type(args[arg]) is not arg_type and not type(args[arg]).__name__ == arg_type:
                     if is_class:
@@ -379,6 +387,7 @@ def create_block_snapshot(save: bool = False, output_path: str = ''):
             block_snapshot[key][func_name] = {
                 "signature": meta["signature"],
                 "docstring": meta["docstring"],
+                "coroutine": meta["coroutine"],
                 "path": f"{func.__module__}.{func.__qualname__}"
             }
     if block_snapshot:
@@ -443,4 +452,23 @@ def get_local_ip():
         ip = '127.0.0.1'
     finally:
         s.close()
-    return ip
\ No newline at end of file
+    return ip
+
+
+def safe_dump(obj):
+    try:
+        json.dumps(obj)
+        return obj
+    except (TypeError, OverflowError):
+        if isinstance(obj, Enum):
+            return obj.value
+        return repr(obj)  # store readable representation
+
+
+def create_module_snapshot(module):
+    classes = inspect.getmembers(module, inspect.isclass)
+    api_variables = {}
+    for i in classes:
+        # globals()[i[0]] = i[1]
+        api_variables[i[0]] = i[1]
+    return api_variables
\ No newline at end of file
